<?php
/**
 * Integrations: Integration
 * Integration abstract class, each integration must extend it
 *
 * @author MS
 */
namespace Freshmail\Core\Integrations;

abstract class Integration
{
  /**
   * @var string option name prefix
   */
  const OPTION_PREFIX = 'f_';

  /**
   * @var string integration name
   */
  public $name = __CLASS__ . ' Integration';

  public function getOptionName(string $option): string
  {
    return $this::OPTION_PREFIX . $option . '_' . __CLASS__;
  }

  /**
   * Checks if integration is active
   *
   * @return bool
   * @author MS
   */
  public function isActive(): bool
  {
    return $this->isEnabled();
  }

  /**
   * Checks if integration is enabled
   *
   * @return bool
   * @author MS
   */
  public function isEnabled(): bool
  {
    return (bool) get_option($this->getOptionName('enabled'), false);
  }

  /**
   * Activates the integration
   *
   * @return bool true if integration was activated
   * @author MS
   */
  public function activate(): bool
  {
    return $this->enable();
  }

  /**
   * Enables the integration
   *
   * @return bool true if integration was enabled
   * @author MS
   */
  public function enable(): bool
  {
    if ($this) {
      return update_option($this->getOptionName('enabled'), true);
    }
    return false;
  }

  /**
   * Deactivates the integration
   *
   * @return bool true if integration was deactivated
   * @author MS
   */
  public function deactivate(): bool
  {
    return $this->disable();
  }

  /**
   * Disables the integration
   *
   * @return bool true if integration was disabled
   * @author MS
   */
  public function disable(): bool
  {
    return update_option($this->getOptionName('enabled'), false);
  }

  /**
   * Returns integration name
   *
   * @return string
   * @author MS
   */
  public function getName(): string
  {
    return $this->name;
  }

  /**
   * Checks if integration can be activated.
   *
   * @return bool true if integration can be activated otherwise false
   */
  public function canBeActivated()
  {
    return $this->canBeEnabled();
  }

  /**
   * Returns integration sections for settings screen.
   * See AdminMenu core module for more info
   *
   * @return array
   */
  abstract public function getSettingsSections(): array;

  /**
   * Returns integration fields for settings screen.
   * See AdminMenu core module for more info
   *
   * @return array
   */
  abstract public function getSettingsFields(): array;

  /**
   * Checks if integration can be enabled.
   *
   * @return bool true if integration can be enabled otherwise false
   */
  abstract public function canBeEnabled(): bool;
}
